#include "Board.h"
#include "dbgu.h"

//* Internal functions

void Wait (unsigned long Speed);
void Init_LED (void);
void Set_LED (unsigned int Led);
void Clean_LED (unsigned int Led);
void Trace_Toggle_LED( unsigned int led) ;

//* external function
extern unsigned enableIRQ();
extern unsigned disableIRQ();
extern unsigned enableFIQ();
extern unsigned disableFIQ();
extern unsigned _bootsize;

volatile unsigned long count = 0;
#define AT91B_MASTER_CLOCK	((18432000*73/14)/2)   // Output PLL Clock (48 MHz)

#define SPEED 		(MCKKHz/10)
unsigned int LedSpeed = SPEED *50 ;

//*--------------------------------------------------------------------------------------
//* Function Name       : Wait
//* Object              : Software waiting loop
//* Input Parameters    : Speed
//* Output Parameters   : none
//*--------------------------------------------------------------------------------------
void Wait1 (unsigned long Speed)
{
    unsigned long waiting_time;
    
    for(waiting_time = 0; waiting_time < Speed; waiting_time++);
}

//*--------------------------------------------------------------------------------------
//* Function Name       : wait
//* Object              : Software waiting loop
//* Input Parameters    : none. Waiting time is defined by the global variable LedSpeed.
//* Output Parameters   : none
//*--------------------------------------------------------------------------------------
static void wait ( void )
{//* Begin
    volatile unsigned int waiting_time ;
 //   change_speed () ;
    for(waiting_time = 0; waiting_time < LedSpeed; waiting_time++) ;
}//* End

//*----------------------------------------------------------------------------
//* \fn    Init_Ports
//* \brief This function Initializs the LEDs and other ports
//*----------------------------------------------------------------------------
void Init_Ports (void)
{
    // Led init
    //Initializes only LED1-LED3
    AT91PS_PIO pLed = AT91C_BASE_PIOA;
    // First, enable the clock of the PIOA
    AT91F_PMC_EnablePeriphClock ( AT91C_BASE_PMC, 1 << AT91C_ID_PIOA ) ;
    /* Configure the pin in output */
    pLed->PIO_OER = (LED1 | LED2 | LED3 | AT91C_PIO_PA21);
    /* Set the PIO controller in PIO mode instead of peripheral mode */
    pLed->PIO_PER = (LED1 | LED2 | LED3 | AT91C_PIO_PA21);
    /* Disable pull-up */
    pLed->PIO_PPUDR = (LED1 | LED2 | LED3 | AT91C_PIO_PA21);
  
    /* Set the default state for the led */
    pLed->PIO_SODR = LED1 | LED2 | LED3;
}

//*----------------------------------------------------------------------------
//* Function Name       : Trace_Toggel_LED
//* Object              : Toggel a LED
//*----------------------------------------------------------------------------
void Trace_Toggle_LED (unsigned int Led)
{
    if ( (AT91F_PIO_GetInput(AT91C_BASE_PIOA) & Led ) == Led )
    {
        AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, Led );
    }
    else
    {
        AT91F_PIO_SetOutput( AT91C_BASE_PIOA, Led );
    }
}

//*----------------------------------------------------------------------------
//* Function Name       : Set_LED
//* Object              : Set a LED
//*----------------------------------------------------------------------------
void Set_LED (unsigned int Led)
{
    AT91F_PIO_SetOutput( AT91C_BASE_PIOA, Led );
}

//*----------------------------------------------------------------------------
//* Function Name       : Clear_LED
//* Object              : Clear a LED
//*----------------------------------------------------------------------------
void Clear_LED (unsigned int Led)
{
    AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, Led );
}

void timer_handler(void)
{
  volatile unsigned long dummy;
  /* Clear status bit */
  dummy = AT91C_BASE_TC0->TC_SR;

  count++;
  if (count > 30) {
    count = 0;
    Clear_LED(LED2);
    AT91F_DBGU_Printk("Timer elapsed, starting application...\n\r");
    //Run the application
    //((funct)0x00104000)();
  } else {
    Set_LED(LED2);
  }

  /* Toggle LED state */
  /*if (BASE_PIO_LED->PIO_ODSR & LED_B) {
    BASE_PIO_LED->PIO_CODR = LED_B;
  } else {
    BASE_PIO_LED->PIO_SODR = LED_B;
  }*/
}

static void configure_tc(void)
{
    volatile unsigned long dummy;

    /* Enable periph clock for the PIO controller */
    AT91C_BASE_PMC->PMC_PCER = (1 << AT91C_ID_TC0);

    /* Enable the periph */
    /* Disable the clock and the interrupts */
    AT91C_BASE_TC0->TC_CCR = AT91C_TC_CLKDIS;
    AT91C_BASE_TC0->TC_IDR = 0xFFFFFFFF;

    /* Clear status bit */
    dummy = AT91C_BASE_TC0->TC_SR;

    /* Set the Mode of the Timer Counter */
    AT91C_BASE_TC0->TC_CMR = AT91C_TC_CLKS_TIMER_DIV5_CLOCK | AT91C_TC_CPCTRG;
    AT91C_BASE_TC0->TC_RC = AT91B_MASTER_CLOCK >> 12;  /* MCKR divided by 1024 * 4 */

    /* Enable interrupts */
    /* Disable the interrupt on the interrupt controller */
    AT91C_BASE_AIC->AIC_IDCR = (1 << AT91C_ID_TC0);
    /* Save the interrupt handler routine pointer and the interrupt priority */
    AT91C_BASE_AIC->AIC_SVR[AT91C_ID_TC0] = (unsigned long) timer_handler;
    /* Store the Source Mode Register */
    AT91C_BASE_AIC->AIC_SMR[AT91C_ID_TC0] = AT91C_AIC_SRCTYPE_INT_HIGH_LEVEL | AT91C_AIC_PRIOR_LOWEST;
    /* Clear the interrupt on the interrupt controller */
    AT91C_BASE_AIC->AIC_ICCR = (1 << AT91C_ID_TC0);

    AT91C_BASE_TC0->TC_IER = AT91C_TC_CPCS;

    /* Enable the interrupt on the interrupt controller */
    AT91C_BASE_AIC->AIC_IECR = (1 << AT91C_ID_TC0);

    /* Enable the LED B timer */
    //if (led_b_active) {
      /* Clock is started */
      AT91C_BASE_TC0->TC_CCR = AT91C_TC_CLKEN;
      /* Counter is reset and the clock is started */
      AT91C_BASE_TC0->TC_CCR = AT91C_TC_SWTRG;
    //}
}

//*--------------------------------------------------------------------------------------
//* Function Name       : main
//* Object              :
//*--------------------------------------------------------------------------------------
int main ( void )
{
    char message[10];
    
    enableIRQ();
    enableFIQ();
    
    configure_tc();
    Init_Ports ();

    Clear_LED(LED1);

    //AT91F_AIC_DisableIt (AT91C_BASE_AIC, AT91C_ID_SYS);
    AT91F_DBGU_Init();
    //siprintf(message, "Bootsize: %lu\n\r", _bootsize);
    //AT91F_DBGU_Frame(message);

    AT91F_DBGU_Printk("Application running...\n\r");

    while (1)
    {
    }
}
